@@ -1,4 +1,4 @@
-/*
+/* -*- c-basic-offset: 8; indent-tabs-mode: t -*-
  * panel-addto.c:
  *
  * Copyright (C) 2004 Vincent Untz
@@ -34,7 +34,6 @@
 
 #include "launcher.h"
 #include "panel.h"
-#include "drawer.h"
 #include "panel-applets-manager.h"
 #include "panel-applet-frame.h"
 #include "panel-action-button.h"
@@ -43,11 +42,12 @@
 #include "panel-toplevel.h"
 #include "panel-menu-button.h"
 #include "panel-globals.h"
+#include "panel-layout.h"
 #include "panel-lockdown.h"
 #include "panel-util.h"
-#include "panel-profile.h"
 #include "panel-addto.h"
 #include "panel-icon-names.h"
+#include "panel-user-menu.h"
 
 typedef struct {
 	PanelWidget *panel_widget;
@@ -74,7 +74,7 @@ typedef struct {
 
 	guint         name_notify;
 
-	int           insertion_position;
+	PanelObjectPackType insert_pack_type;
 } PanelAddtoDialog;
 
 static GQuark panel_addto_dialog_quark = 0;
@@ -88,20 +88,20 @@ typedef enum {
 	PANEL_ADDTO_MENU,
 	PANEL_ADDTO_MENUBAR,
 	PANEL_ADDTO_SEPARATOR,
-	PANEL_ADDTO_DRAWER
+	PANEL_ADDTO_USER_MENU
 } PanelAddtoItemType;
 
 typedef struct {
 	PanelAddtoItemType     type;
 	char                  *name;
 	char                  *description;
-	char                  *icon;
+	GIcon                 *icon;
 	PanelActionButtonType  action_type;
 	char                  *launcher_path;
 	char                  *menu_filename;
 	char                  *menu_path;
 	char                  *iid;
-	gboolean               static_data;
+	gboolean               static_strings;
 } PanelAddtoItemInfo;
 
 typedef struct {
@@ -109,79 +109,6 @@ typedef struct {
 	PanelAddtoItemInfo  item_info;
 } PanelAddtoAppList;
 
-static PanelAddtoItemInfo special_addto_items [] = {
-
-	{ PANEL_ADDTO_LAUNCHER_NEW,
-	  N_("Custom Application Launcher"),
-	  N_("Create a new launcher"),
-	  PANEL_ICON_LAUNCHER,
-	  PANEL_ACTION_NONE,
-	  NULL,
-	  NULL,
-	  NULL,
-	  "LAUNCHER:ASK",
-	  TRUE },
-
-	{ PANEL_ADDTO_LAUNCHER_MENU,
-	  N_("Application Launcher..."),
-	  N_("Copy a launcher from the applications menu"),
-	  PANEL_ICON_LAUNCHER,
-	  PANEL_ACTION_NONE,
-	  NULL,
-	  NULL,
-	  NULL,
-	  "LAUNCHER:MENU",
-	  TRUE }
-
-};
-
-static PanelAddtoItemInfo internal_addto_items [] = {
-
-	{ PANEL_ADDTO_MENU,
-	  N_("Main Menu"),
-	  N_("The main GNOME menu"),
-	  PANEL_ICON_MAIN_MENU,
-	  PANEL_ACTION_NONE,
-	  NULL,
-	  NULL,
-	  NULL,
-	  "MENU:MAIN",
-	  TRUE },
-
-	{ PANEL_ADDTO_MENUBAR,
-	  N_("Menu Bar"),
-	  N_("A custom menu bar"),
-	  PANEL_ICON_MAIN_MENU,
-	  PANEL_ACTION_NONE,
-	  NULL,
-	  NULL,
-	  NULL,
-	  "MENUBAR:NEW",
-	  TRUE },
-
-	{ PANEL_ADDTO_SEPARATOR,
-	  N_("Separator"),
-	  N_("A separator to organize the panel items"),
-	  PANEL_ICON_SEPARATOR,
-	  PANEL_ACTION_NONE,
-	  NULL,
-	  NULL,
-	  NULL,
-	  "SEPARATOR:NEW",
-	  TRUE },
-
-	{ PANEL_ADDTO_DRAWER,
-	  N_("Drawer"),
-	  N_("A pop out drawer to store other items in"),
-	  PANEL_ICON_DRAWER,
-	  PANEL_ACTION_NONE,
-	  NULL,
-	  NULL,
-	  NULL,
-	  "DRAWER:NEW",
-	  TRUE }
-};
-
 enum {
 	COLUMN_ICON,
 	COLUMN_TEXT,
@@ -208,22 +135,60 @@ panel_addto_applet_info_sort_func (PanelAddtoItemInfo *a,
 	return g_utf8_collate (a->name, b->name);
 }
 
+static GSList *
+prepend_internal_applets (GSList *list)
+{
+	PanelAddtoItemInfo *internal;
+
+	internal = g_new0 (PanelAddtoItemInfo, 1);
+	internal->type = PANEL_ADDTO_MENU;
+	internal->name = _("Main Menu");
+	internal->description = _("The main GNOME menu");
+	internal->icon = g_themed_icon_new (PANEL_ICON_MAIN_MENU);
+	internal->action_type = PANEL_ACTION_NONE;
+	internal->iid = "MENU:MAIN";
+	internal->static_strings = TRUE;
+	list = g_slist_prepend (list, internal);
+	
+	internal = g_new0 (PanelAddtoItemInfo, 1);
+	internal->type = PANEL_ADDTO_MENUBAR;
+	internal->name = _("Menu Bar");
+	internal->description = _("A custom menu bar");
+	internal->icon = g_themed_icon_new (PANEL_ICON_MAIN_MENU);
+	internal->action_type = PANEL_ACTION_NONE;
+	internal->iid = "MENUBAR:NEW";
+	internal->static_strings = TRUE;
+	list = g_slist_prepend (list, internal);
+
+	internal = g_new0 (PanelAddtoItemInfo, 1);
+	internal->type = PANEL_ADDTO_SEPARATOR;
+	internal->name = _("Separator");
+	internal->description = _("A separator to organize the panel items");
+	internal->icon = g_themed_icon_new (PANEL_ICON_SEPARATOR);
+	internal->action_type = PANEL_ACTION_NONE;
+	internal->iid = "SEPARATOR:NEW";
+	internal->static_strings = TRUE;
+	list = g_slist_prepend (list, internal);
+
+	internal = g_new0 (PanelAddtoItemInfo, 1);
+	internal->type = PANEL_ADDTO_USER_MENU;
+	internal->name = _("User menu");
+	internal->description = _("Menu to change your settings and your online status");
+	internal->icon = g_themed_icon_new (PANEL_ICON_USER_AVAILABLE);
+	internal->action_type = PANEL_ACTION_NONE;
+	internal->iid = "USERMENU:NEW";
+	internal->static_strings = TRUE;
+	list = g_slist_prepend (list, internal);
+
+	return list;
+}
+
 static GSList *
 panel_addto_prepend_internal_applets (GSList *list)
 {
-	static gboolean translated = FALSE;
-	int             i;
+	int i;
 
-	for (i = 0; i < G_N_ELEMENTS (internal_addto_items); i++) {
-		if (!translated) {
-			internal_addto_items [i].name        = _(internal_addto_items [i].name);
-			internal_addto_items [i].description = _(internal_addto_items [i].description);
-		}
-
-                list = g_slist_prepend (list, &internal_addto_items [i]);
-        }
-
-	translated = TRUE;
+	list = prepend_internal_applets (list);
 
 	for (i = PANEL_ACTION_LOCK; i < PANEL_ACTION_LAST; i++) {
 		PanelAddtoItemInfo *info;
@@ -234,11 +199,12 @@ panel_addto_prepend_internal_applets (GSList *list)
 		info              = g_new0 (PanelAddtoItemInfo, 1);
 		info->type        = PANEL_ADDTO_ACTION;
 		info->action_type = i;
-		info->name        = g_strdup (panel_action_get_text (i));
-		info->description = g_strdup (panel_action_get_tooltip (i));
-		info->icon        = g_strdup (panel_action_get_icon_name (i));
-		info->iid         = g_strdup (panel_action_get_drag_id (i));
-		info->static_data = FALSE;
+		info->name        = (char *) panel_action_get_text (i);
+		info->description = (char *) panel_action_get_tooltip (i);
+		if (panel_action_get_icon_name (i) != NULL)
+			info->icon = g_themed_icon_new (panel_action_get_icon_name (i));
+		info->iid         = (char *) panel_action_get_drag_id (i);
+		info->static_strings = TRUE;
 
 		list = g_slist_prepend (list, info);
 	}
@@ -266,18 +232,6 @@ panel_addto_make_text (const char *name,
 	return result;
 }
 
-#define ICON_SIZE 32
-
-static GdkPixbuf *
-panel_addto_make_pixbuf (const char *filename)
-{
-	//FIXME: size shouldn't be fixed but should depend on the font size
-	return panel_load_icon (gtk_icon_theme_get_default (),
-				filename,
-				ICON_SIZE, ICON_SIZE, ICON_SIZE,
-				NULL);
-}
-
 static void  
 panel_addto_drag_data_get_cb (GtkWidget        *widget,
 			      GdkDragContext   *context,
@@ -287,8 +241,8 @@ panel_addto_drag_data_get_cb (GtkWidget        *widget,
 			      const char       *string)
 {
 	gtk_selection_data_set (selection_data,
-				selection_data->target, 8, (guchar *) string,
-				strlen (string));
+				gtk_selection_data_get_target (selection_data), 8,
+				(guchar *) string, strlen (string));
 }
 
 static void
@@ -301,7 +255,7 @@ panel_addto_drag_begin_cb (GtkWidget      *widget,
 	GtkTreePath  *path;
 	GtkTreeIter   iter;
 	GtkTreeIter   filter_iter;
-	GdkPixbuf    *pixbuf;
+	GIcon        *gicon;
 
 	filter_model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
 	   
@@ -313,11 +267,11 @@ panel_addto_drag_begin_cb (GtkWidget      *widget,
 
 	child_model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (filter_model));
 	gtk_tree_model_get (child_model, &iter,
-	                    COLUMN_ICON, &pixbuf,
+	                    COLUMN_ICON, &gicon,
 	                    -1);
 
-	gtk_drag_set_icon_pixbuf (context, pixbuf, 0, 0);
-	g_object_unref (pixbuf);
+	gtk_drag_set_icon_gicon (context, gicon, 0, 0);
+	g_object_unref (gicon);
 }
 
 static void
@@ -325,7 +279,7 @@ panel_addto_setup_drag (GtkTreeView          *tree_view,
 			const GtkTargetEntry *target,
 			const char           *text)
 {
-	if (!text || panel_lockdown_get_locked_down ())
+	if (!text || panel_lockdown_get_panels_locked_down_s ())
 		return;
 	
 	gtk_tree_view_enable_model_drag_source (GTK_TREE_VIEW (tree_view),
@@ -400,7 +354,9 @@ panel_addto_query_applets (GSList *list)
 		description = panel_applet_info_get_description (info);
 		icon = panel_applet_info_get_icon (info);
 
-		if (!name || panel_lockdown_is_applet_disabled (iid)) {
+		if (!name ||
+		    panel_lockdown_is_applet_disabled (panel_lockdown_get (),
+						       iid)) {
 			continue;
 		}
 
@@ -408,9 +364,10 @@ panel_addto_query_applets (GSList *list)
 		applet->type = PANEL_ADDTO_APPLET;
 		applet->name = g_strdup (name);
 		applet->description = g_strdup (description);
-		applet->icon = g_strdup (icon);
+		if (icon)
+			applet->icon = g_themed_icon_new (icon);
 		applet->iid = g_strdup (iid);
-		applet->static_data = FALSE;
+		applet->static_strings = FALSE;
 
 		list = g_slist_prepend (list, applet);
 	}
@@ -426,7 +383,6 @@ panel_addto_append_item (PanelAddtoDialog *dialog,
 			 PanelAddtoItemInfo *applet)
 {
 	char *text;
-	GdkPixbuf *pixbuf;
 	GtkTreeIter iter;
 
 	if (applet == NULL) {
@@ -438,27 +394,18 @@ panel_addto_append_item (PanelAddtoDialog *dialog,
 				    COLUMN_SEARCH, NULL,
 				    -1);
 	} else {
-		pixbuf = NULL;
-
-		if (applet->icon != NULL) {
-			pixbuf = panel_addto_make_pixbuf (applet->icon);
-		}
-
 		gtk_list_store_append (model, &iter);
 
 		text = panel_addto_make_text (applet->name,
 					      applet->description);
 
 		gtk_list_store_set (model, &iter,
-				    COLUMN_ICON, pixbuf,
+				    COLUMN_ICON, applet->icon,
 				    COLUMN_TEXT, text,
 				    COLUMN_DATA, applet,
 				    COLUMN_SEARCH, applet->name,
 				    -1);
 
-		if (pixbuf)
-			g_object_unref (pixbuf);
-
 		g_free (text);
 	}
 }
@@ -467,23 +414,29 @@ static void
 panel_addto_append_special_applets (PanelAddtoDialog *dialog,
 				    GtkListStore *model)
 {
-	static gboolean translated = FALSE;
-	int i;
-	
-	for (i = 0; i < G_N_ELEMENTS (special_addto_items); i++) {
-		if (!translated) {
-			special_addto_items [i].name = _(special_addto_items [i].name);
-			special_addto_items [i].description = _(special_addto_items [i].description);
-		}
+	PanelAddtoItemInfo *special;
 
-		if (special_addto_items [i].type == PANEL_ADDTO_LAUNCHER_NEW
-		    && panel_lockdown_get_disable_command_line ())
-			continue;
-		
-		panel_addto_append_item (dialog, model, &special_addto_items [i]);
+	if (!panel_lockdown_get_disable_command_line_s ()) {
+		special = g_new0 (PanelAddtoItemInfo, 1);
+		special->type = PANEL_ADDTO_LAUNCHER_NEW;
+		special->name = _("Custom Application Launcher");
+		special->description = _("Create a new launcher");
+		special->icon = g_themed_icon_new (PANEL_ICON_LAUNCHER);
+		special->action_type = PANEL_ACTION_NONE;
+		special->iid = "LAUNCHER:ASK";
+		special->static_strings = TRUE;
+		panel_addto_append_item (dialog, model, special);
 	}
-	
-	translated = TRUE;
+
+	special = g_new0 (PanelAddtoItemInfo, 1);
+	special->type = PANEL_ADDTO_LAUNCHER_MENU;
+	special->name = _("Application Launcher...");
+	special->description = _("Copy a launcher from the applications menu");
+	special->icon = g_themed_icon_new (PANEL_ICON_LAUNCHER);
+	special->action_type = PANEL_ACTION_NONE;
+	special->iid = "LAUNCHER:MENU";
+	special->static_strings = TRUE;
+	panel_addto_append_item (dialog, model, special);
 }
 
 static void
@@ -495,7 +448,7 @@ panel_addto_make_applet_model (PanelAddtoDialog *dialog)
 	if (dialog->filter_applet_model != NULL)
 		return;
 
-	if (panel_profile_id_lists_are_writable ()) {
+	if (panel_layout_is_writable ()) {
 		dialog->applet_list = panel_addto_query_applets (dialog->applet_list);
 		dialog->applet_list = panel_addto_prepend_internal_applets (dialog->applet_list);
 	}
@@ -504,12 +457,12 @@ panel_addto_make_applet_model (PanelAddtoDialog *dialog)
 					    (GCompareFunc) panel_addto_applet_info_sort_func);
 
 	model = gtk_list_store_new (NUMBER_COLUMNS,
-				    GDK_TYPE_PIXBUF,
+				    G_TYPE_ICON,
 				    G_TYPE_STRING,
 				    G_TYPE_POINTER,
 				    G_TYPE_STRING);
 
-	if (panel_profile_id_lists_are_writable ()) {
+	if (panel_layout_is_writable ()) {
 		panel_addto_append_special_applets (dialog, model);
 		if (dialog->applet_list)
 			panel_addto_append_item (dialog, model, NULL);
@@ -526,9 +479,17 @@ panel_addto_make_applet_model (PanelAddtoDialog *dialog)
 						dialog, NULL);
 }
 
-static void panel_addto_make_application_list (GSList             **parent_list,
-					       GMenuTreeDirectory  *directory,
-					       const char          *filename);
+typedef enum {
+	PANEL_ADDTO_MENU_SHOW_DIRECTORIES = 1 << 0,
+	PANEL_ADDTO_MENU_SHOW_ENTRIES     = 1 << 1,
+	PANEL_ADDTO_MENU_SHOW_ALIAS       = 1 << 2
+#define PANEL_ADDTO_MENU_SHOW_ALL (PANEL_ADDTO_MENU_SHOW_DIRECTORIES | PANEL_ADDTO_MENU_SHOW_ENTRIES | PANEL_ADDTO_MENU_SHOW_ALIAS)
+} PanelAddtoMenuShowFlags;
+
+static void panel_addto_make_application_list (GSList                  **parent_list,
+					       GMenuTreeDirectory       *directory,
+					       const char               *filename,
+					       PanelAddtoMenuShowFlags   show_flags);
 
 static void
 panel_addto_prepend_directory (GSList             **parent_list,
@@ -542,10 +503,10 @@ panel_addto_prepend_directory (GSList             **parent_list,
 	data->item_info.type          = PANEL_ADDTO_MENU;
 	data->item_info.name          = g_strdup (gmenu_tree_directory_get_name (directory));
 	data->item_info.description   = g_strdup (gmenu_tree_directory_get_comment (directory));
-	data->item_info.icon          = g_strdup (gmenu_tree_directory_get_icon (directory));
+	data->item_info.icon          = g_object_ref (gmenu_tree_directory_get_icon (directory));
 	data->item_info.menu_filename = g_strdup (filename);
 	data->item_info.menu_path     = gmenu_tree_directory_make_path (directory, NULL);
-	data->item_info.static_data   = FALSE;
+	data->item_info.static_strings = FALSE;
 
 	/* We should set the iid here to something and do
 	 * iid = g_strdup_printf ("MENU:%s", tfr->name)
@@ -557,7 +518,9 @@ panel_addto_prepend_directory (GSList             **parent_list,
 
 	*parent_list = g_slist_prepend (*parent_list, data);
 			
-	panel_addto_make_application_list (&data->children, directory, filename);
+	/* We always want to show everything in non-root directories */
+	panel_addto_make_application_list (&data->children, directory,
+					   filename, PANEL_ADDTO_MENU_SHOW_ALL);
 }
 
 static void
@@ -566,15 +529,18 @@ panel_addto_prepend_entry (GSList         **parent_list,
 			   const char      *filename)
 {
 	PanelAddtoAppList *data;
+	GAppInfo *app_info;
 
 	data = g_new0 (PanelAddtoAppList, 1);
 
+	app_info = G_APP_INFO (gmenu_tree_entry_get_app_info (entry));
+
 	data->item_info.type          = PANEL_ADDTO_LAUNCHER;
-	data->item_info.name          = g_strdup (gmenu_tree_entry_get_display_name (entry));
-	data->item_info.description   = g_strdup (gmenu_tree_entry_get_comment (entry));
-	data->item_info.icon          = g_strdup (gmenu_tree_entry_get_icon (entry));
+	data->item_info.name          = g_strdup (g_app_info_get_display_name (app_info));
+	data->item_info.description   = g_strdup (g_app_info_get_description (app_info));
+	data->item_info.icon          = g_object_ref (g_app_info_get_icon (app_info));
 	data->item_info.launcher_path = g_strdup (gmenu_tree_entry_get_desktop_file_path (entry));
-	data->item_info.static_data   = FALSE;
+	data->item_info.static_strings = FALSE;
 
 	*parent_list = g_slist_prepend (*parent_list, data);
 }
@@ -584,62 +550,70 @@ panel_addto_prepend_alias (GSList         **parent_list,
 			   GMenuTreeAlias  *alias,
 			   const char      *filename)
 {
-	GMenuTreeItem *aliased_item;
-
-	aliased_item = gmenu_tree_alias_get_item (alias);
-
-	switch (gmenu_tree_item_get_type (aliased_item)) {
-	case GMENU_TREE_ITEM_DIRECTORY:
+	switch (gmenu_tree_alias_get_aliased_item_type (alias)) {
+	case GMENU_TREE_ITEM_DIRECTORY: {
+		GMenuTreeDirectory *directory = gmenu_tree_alias_get_aliased_directory (alias);
 		panel_addto_prepend_directory (parent_list,
-					       GMENU_TREE_DIRECTORY (aliased_item),
+					       directory,
 					       filename);
+		gmenu_tree_item_unref (directory);
 		break;
-
-	case GMENU_TREE_ITEM_ENTRY:
+	}
+	case GMENU_TREE_ITEM_ENTRY: {
+		GMenuTreeEntry *entry = gmenu_tree_alias_get_aliased_entry (alias);
 		panel_addto_prepend_entry (parent_list,
-					   GMENU_TREE_ENTRY (aliased_item),
+					   entry,
 					   filename);
+		gmenu_tree_item_unref (entry);
 		break;
-
+	}
 	default:
 		break;
 	}
-
-	gmenu_tree_item_unref (aliased_item);
 }
 
 static void
-panel_addto_make_application_list (GSList             **parent_list,
-				   GMenuTreeDirectory  *directory,
-				   const char          *filename)
+panel_addto_make_application_list (GSList                  **parent_list,
+				   GMenuTreeDirectory       *directory,
+				   const char               *filename,
+				   PanelAddtoMenuShowFlags   show_flags)
 {
-	GSList *items;
-	GSList *l;
+	GMenuTreeIter *iter;
+	GMenuTreeItemType next_type;
 
-	items = gmenu_tree_directory_get_contents (directory);
+	iter = gmenu_tree_directory_iter (directory);
 
-	for (l = items; l; l = l->next) {
-		switch (gmenu_tree_item_get_type (l->data)) {
+	while ((next_type = gmenu_tree_iter_next (iter)) != GMENU_TREE_ITEM_INVALID) {
+		switch (next_type) {
 		case GMENU_TREE_ITEM_DIRECTORY:
-			panel_addto_prepend_directory (parent_list, l->data, filename);
+			if (show_flags & PANEL_ADDTO_MENU_SHOW_DIRECTORIES) {
+				GMenuTreeDirectory *dir = gmenu_tree_iter_get_directory (iter);
+				panel_addto_prepend_directory (parent_list, dir, filename);
+				gmenu_tree_item_unref (dir);
+			}
 			break;
 
 		case GMENU_TREE_ITEM_ENTRY:
-			panel_addto_prepend_entry (parent_list, l->data, filename);
+			if (show_flags & PANEL_ADDTO_MENU_SHOW_ENTRIES) {
+				GMenuTreeEntry *entry = gmenu_tree_iter_get_entry (iter);
+				panel_addto_prepend_entry (parent_list, entry, filename);
+				gmenu_tree_item_unref (entry);
+			}
 			break;
 
 		case GMENU_TREE_ITEM_ALIAS:
-			panel_addto_prepend_alias (parent_list, l->data, filename);
+			if (show_flags & PANEL_ADDTO_MENU_SHOW_ALIAS) {
+				GMenuTreeAlias *alias = gmenu_tree_iter_get_alias (iter);
+				panel_addto_prepend_alias (parent_list, alias, filename);
+				gmenu_tree_item_unref (alias);
+			}
 			break;
 
 		default:
 			break;
 		}
-
-		gmenu_tree_item_unref (l->data);
 	}
-
-	g_slist_free (items);
+	gmenu_tree_iter_unref (iter);
 
 	*parent_list = g_slist_reverse (*parent_list);
 }
@@ -652,7 +626,6 @@ panel_addto_populate_application_model (GtkTreeStore *store,
 	PanelAddtoAppList *data;
 	GtkTreeIter        iter;
 	char              *text;
-	GdkPixbuf         *pixbuf;
 	GSList            *app;
 
 	for (app = app_list; app != NULL; app = app->next) {
@@ -661,17 +634,13 @@ panel_addto_populate_application_model (GtkTreeStore *store,
 
 		text = panel_addto_make_text (data->item_info.name,
 					      data->item_info.description);
-		pixbuf = panel_addto_make_pixbuf (data->item_info.icon);
 		gtk_tree_store_set (store, &iter,
-				    COLUMN_ICON, pixbuf,
+				    COLUMN_ICON, data->item_info.icon,
 				    COLUMN_TEXT, text,
 				    COLUMN_DATA, &(data->item_info),
 				    COLUMN_SEARCH, data->item_info.name,
 				    -1);
 
-		if (pixbuf)
-			g_object_unref (pixbuf);
-
 		g_free (text);
 
 		if (data->children != NULL) 
@@ -692,28 +661,38 @@ panel_addto_make_application_model (PanelAddtoDialog *dialog)
 		return;
 
 	store = gtk_tree_store_new (NUMBER_COLUMNS,
-				    GDK_TYPE_PIXBUF,
+				    G_TYPE_ICON,
 				    G_TYPE_STRING,
 				    G_TYPE_POINTER,
 				    G_TYPE_STRING);
 
-	tree = gmenu_tree_lookup ("applications.menu", GMENU_TREE_FLAGS_NONE);
-	gmenu_tree_set_sort_key (tree, GMENU_TREE_SORT_DISPLAY_NAME);
+	tree = gmenu_tree_new ("applications.menu", GMENU_TREE_FLAGS_SORT_DISPLAY_NAME);
 
-	if ((root = gmenu_tree_get_root_directory (tree))) {
+	if (!gmenu_tree_load_sync (tree, NULL)) {
+		g_object_unref (tree);
+		tree = NULL;
+	}
+
+	if (tree != NULL && (root = gmenu_tree_get_root_directory (tree))) {
 		panel_addto_make_application_list (&dialog->application_list,
-						   root, "applications.menu");
+						   root, "applications.menu",
+						   PANEL_ADDTO_MENU_SHOW_ALL);
 		panel_addto_populate_application_model (store, NULL, dialog->application_list);
 
 		gmenu_tree_item_unref (root);
 	}
 
-	gmenu_tree_unref (tree);
+	if (tree != NULL)
+		g_object_unref (tree);
 
-	tree = gmenu_tree_lookup ("settings.menu", GMENU_TREE_FLAGS_NONE);
-	gmenu_tree_set_sort_key (tree, GMENU_TREE_SORT_DISPLAY_NAME);
+	tree = gmenu_tree_new ("gnomecc.menu", GMENU_TREE_FLAGS_SORT_DISPLAY_NAME);
 
-	if ((root = gmenu_tree_get_root_directory (tree))) {
+	if (!gmenu_tree_load_sync (tree, NULL)) {
+		g_object_unref (tree);
+		tree = NULL;
+	}
+
+	if (tree != NULL && (root = gmenu_tree_get_root_directory (tree))) {
 		GtkTreeIter iter;
 
 		gtk_tree_store_append (store, &iter, NULL);
@@ -724,15 +703,19 @@ panel_addto_make_application_model (PanelAddtoDialog *dialog)
 				    COLUMN_SEARCH, NULL,
 				    -1);
 
+		/* The gnome-control-center shell does not display toplevel
+		 * entries that are not directories, so do the same. */
 		panel_addto_make_application_list (&dialog->settings_list,
-						   root, "settings.menu");
+						   root, "gnomecc.menu",
+						   PANEL_ADDTO_MENU_SHOW_DIRECTORIES);
 		panel_addto_populate_application_model (store, NULL,
 							dialog->settings_list);
 
 		gmenu_tree_item_unref (root);
 	}
 
-	gmenu_tree_unref (tree);
+	if (tree != NULL)
+		g_object_unref (tree);
 
 	dialog->application_model = GTK_TREE_MODEL (store);
 	dialog->filter_application_model = gtk_tree_model_filter_new (GTK_TREE_MODEL (dialog->application_model),
@@ -746,17 +729,24 @@ static void
 panel_addto_add_item (PanelAddtoDialog   *dialog,
 	 	      PanelAddtoItemInfo *item_info)
 {
+	int pack_index;
+
 	g_assert (item_info != NULL);
 
+	pack_index = panel_widget_get_new_pack_index (dialog->panel_widget,
+						      dialog->insert_pack_type);
+
 	switch (item_info->type) {
 	case PANEL_ADDTO_APPLET:
 		panel_applet_frame_create (dialog->panel_widget->toplevel,
-					   dialog->insertion_position,
+					   dialog->insert_pack_type,
+					   pack_index,
 					   item_info->iid);
 		break;
 	case PANEL_ADDTO_ACTION:
 		panel_action_button_create (dialog->panel_widget->toplevel,
-					    dialog->insertion_position,
+					    dialog->insert_pack_type,
+					    pack_index,
 					    item_info->action_type);
 		break;
 	case PANEL_ADDTO_LAUNCHER_MENU:
@@ -764,33 +754,36 @@ panel_addto_add_item (PanelAddtoDialog   *dialog,
 		break;
 	case PANEL_ADDTO_LAUNCHER:
 		panel_launcher_create (dialog->panel_widget->toplevel,
-				       dialog->insertion_position,
+				       dialog->insert_pack_type,
+				       pack_index,
 				       item_info->launcher_path);
 		break;
 	case PANEL_ADDTO_LAUNCHER_NEW:
 		ask_about_launcher (NULL, dialog->panel_widget,
-				    dialog->insertion_position, FALSE);
+				    dialog->insert_pack_type);
 		break;
 	case PANEL_ADDTO_MENU:
 		panel_menu_button_create (dialog->panel_widget->toplevel,
-					  dialog->insertion_position,
+					  dialog->insert_pack_type,
+					  pack_index,
 					  item_info->menu_filename,
 					  item_info->menu_path,
-					  item_info->menu_path != NULL,
 					  item_info->name);
 		break;
 	case PANEL_ADDTO_MENUBAR:
 		panel_menu_bar_create (dialog->panel_widget->toplevel,
-				       dialog->insertion_position);
+				       dialog->insert_pack_type,
+				       pack_index);
 		break;
 	case PANEL_ADDTO_SEPARATOR:
 		panel_separator_create (dialog->panel_widget->toplevel,
-					dialog->insertion_position);
+					dialog->insert_pack_type,
+					pack_index);
 		break;
-	case PANEL_ADDTO_DRAWER:
-		panel_drawer_create (dialog->panel_widget->toplevel,
-				     dialog->insertion_position,
-				     NULL, FALSE, NULL);
+	case PANEL_ADDTO_USER_MENU:
+		panel_user_menu_create (dialog->panel_widget->toplevel,
+					dialog->insert_pack_type,
+					pack_index);
 		break;
 	}
 }
@@ -900,7 +893,15 @@ panel_addto_present_applets (PanelAddtoDialog *dialog)
 static void
 panel_addto_dialog_free_item_info (PanelAddtoItemInfo *item_info)
 {
-	if (item_info == NULL || item_info->static_data)
+	if (item_info == NULL)
+		return;
+
+	/* the GIcon is never static */
+	if (item_info->icon != NULL)
+		g_object_unref (item_info->icon);
+	item_info->icon = NULL;
+
+	if (item_info->static_strings)
 		return;
 
 	if (item_info->name != NULL)
@@ -911,10 +912,6 @@ panel_addto_dialog_free_item_info (PanelAddtoItemInfo *item_info)
 		g_free (item_info->description);
 	item_info->description = NULL;
 
-	if (item_info->icon != NULL)
-		g_free (item_info->icon);
-	item_info->icon = NULL;
-
 	if (item_info->iid != NULL)
 		g_free (item_info->iid);
 	item_info->iid = NULL;
@@ -955,13 +952,11 @@ panel_addto_dialog_free_application_list (GSList *application_list)
 static void
 panel_addto_dialog_free (PanelAddtoDialog *dialog)
 {
-	GConfClient *client;
-	GSList      *item;
-
-	client = panel_gconf_get_client ();
+	GSList *item;
 
 	if (dialog->name_notify)
-		gconf_client_notify_remove (client, dialog->name_notify);
+		g_signal_handler_disconnect (dialog->panel_widget->toplevel,
+					     dialog->name_notify);
 	dialog->name_notify = 0;
 
 	if (dialog->search_text)
@@ -980,10 +975,8 @@ panel_addto_dialog_free (PanelAddtoDialog *dialog)
 		PanelAddtoItemInfo *applet;
 
 		applet = (PanelAddtoItemInfo *) item->data;
-		if (!applet->static_data) {
-			panel_addto_dialog_free_item_info (applet);
-			g_free (applet);
-		}
+		panel_addto_dialog_free_item_info (applet);
+		g_free (applet);
 	}
 	g_slist_free (dialog->applet_list);
 
@@ -1007,64 +1000,41 @@ panel_addto_dialog_free (PanelAddtoDialog *dialog)
 	dialog->application_model = NULL;
 
 	if (dialog->menu_tree)
-		gmenu_tree_unref (dialog->menu_tree);
+		g_object_unref (dialog->menu_tree);
 	dialog->menu_tree = NULL;
 
 	g_free (dialog);
 }
 
 static void
-panel_addto_name_change (PanelAddtoDialog *dialog,
-			 const char       *name)
+panel_addto_name_change (PanelAddtoDialog *dialog)
 {
-	char *title;
-	char *label;
+	const char *name;
+	char       *label;
 
+	name = panel_toplevel_get_name (dialog->panel_widget->toplevel);
 	label = NULL;
 
 	if (!PANEL_GLIB_STR_EMPTY (name))
 		label = g_strdup_printf (_("Find an _item to add to \"%s\":"),
 					 name);
 
-	if (panel_toplevel_get_is_attached (dialog->panel_widget->toplevel)) {
-		title = g_strdup_printf (_("Add to Drawer"));
-		if (label == NULL)
-			label = g_strdup (_("Find an _item to add to the drawer:"));
-	} else {
-		title = g_strdup_printf (_("Add to Panel"));
-		if (label == NULL)
-			label = g_strdup (_("Find an _item to add to the panel:"));
-	}
+	if (label == NULL)
+		label = g_strdup (_("Find an _item to add to the panel:"));
 
-	gtk_window_set_title (GTK_WINDOW (dialog->addto_dialog), title);
-	g_free (title);
+	gtk_window_set_title (GTK_WINDOW (dialog->addto_dialog),
+			      _("Add to Panel"));
 
 	gtk_label_set_text_with_mnemonic (GTK_LABEL (dialog->label), label);
 	g_free (label);
 }
 
 static void
-panel_addto_name_notify (GConfClient      *client,
-			 guint             cnxn_id,
-			 GConfEntry       *entry,
+panel_addto_name_notify (GObject          *gobject,
+			 GParamSpec       *pspec,
 			 PanelAddtoDialog *dialog)
 {
-	GConfValue *value;
-	const char *key;
-	const char *text = NULL;
-
-	key = panel_gconf_basename (gconf_entry_get_key (entry));
-
-	if (strcmp (key, "name"))
-		return;
-
-	value = gconf_entry_get_value (entry);
-
-	if (value && value->type == GCONF_VALUE_STRING)
-		text = gconf_value_get_string (value);
-
-	if (text)
-		panel_addto_name_change (dialog, text);
+	panel_addto_name_change (dialog);
 }
 
 static gboolean
@@ -1185,7 +1155,7 @@ panel_addto_selection_changed (GtkTreeSelection *selection,
 				  TRUE);
 
 	/* only allow dragging applets if we can add applets */
-	if (panel_profile_id_lists_are_writable ()) {
+	if (panel_layout_is_writable ()) {
 		switch (data->type) {
 		case PANEL_ADDTO_LAUNCHER:
 			panel_addto_setup_launcher_drag (GTK_TREE_VIEW (dialog->tree_view),
@@ -1250,7 +1220,7 @@ static PanelAddtoDialog *
 panel_addto_dialog_new (PanelWidget *panel_widget)
 {
 	PanelAddtoDialog *dialog;
-	GtkWidget *vbox;
+	GtkWidget *dialog_vbox;
 	GtkWidget *inner_vbox;
 	GtkWidget *find_hbox;
 	GtkWidget *sw;
@@ -1267,12 +1237,10 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 
 	dialog->panel_widget = panel_widget;
 	dialog->name_notify =
-		panel_profile_toplevel_notify_add (
-			dialog->panel_widget->toplevel,
-			"name",
-			(GConfClientNotifyFunc) panel_addto_name_notify,
-			dialog);
-
+		g_signal_connect (dialog->panel_widget->toplevel,
+				  "notify::name",
+				  G_CALLBACK (panel_addto_name_notify),
+				  dialog);
 
 	dialog->addto_dialog = gtk_dialog_new ();
 	gtk_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
@@ -1287,25 +1255,22 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 			       GTK_STOCK_CLOSE,
 			       GTK_RESPONSE_CLOSE);
 	gtk_widget_set_sensitive (GTK_WIDGET (dialog->add_button), FALSE);
-	gtk_dialog_set_has_separator (GTK_DIALOG (dialog->addto_dialog),
-				      FALSE);
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog->addto_dialog),
 					 PANEL_ADDTO_RESPONSE_ADD);
 
 	gtk_container_set_border_width (GTK_CONTAINER (dialog->addto_dialog), 5);
-	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog->addto_dialog)->vbox), 2);
+
+	dialog_vbox = gtk_dialog_get_content_area (GTK_DIALOG (dialog->addto_dialog));
+	gtk_box_set_spacing (GTK_BOX (dialog_vbox), 12);
+	gtk_container_set_border_width (GTK_CONTAINER (dialog_vbox), 5);
+
 	g_signal_connect (G_OBJECT (dialog->addto_dialog), "response",
 			  G_CALLBACK (panel_addto_dialog_response), dialog);
 	g_signal_connect (dialog->addto_dialog, "destroy",
 			  G_CALLBACK (panel_addto_dialog_destroy), dialog);
 
-	vbox = gtk_vbox_new (FALSE, 12);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
-	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog->addto_dialog)->vbox),
-			   vbox);
-
 	inner_vbox = gtk_vbox_new (FALSE, 6);
-	gtk_box_pack_start (GTK_BOX (vbox), inner_vbox, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (dialog_vbox), inner_vbox, TRUE, TRUE, 0);
 
 	find_hbox = gtk_hbox_new (FALSE, 6);
 	gtk_box_pack_start (GTK_BOX (inner_vbox), find_hbox, FALSE, FALSE, 0);
@@ -1345,12 +1310,13 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 	renderer = g_object_new (GTK_TYPE_CELL_RENDERER_PIXBUF,
 				 "xpad", 4,
 				 "ypad", 4,
+				 "stock-size", GTK_ICON_SIZE_DND,
 				 NULL);
 
 	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (dialog->tree_view),
 						     -1, NULL,
 						     renderer,
-						     "pixbuf", COLUMN_ICON,
+						     "gicon", COLUMN_ICON,
 						     NULL);
 	renderer = gtk_cell_renderer_text_new ();
 	g_object_set (renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
@@ -1387,14 +1353,13 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 
 	gtk_container_add (GTK_CONTAINER (sw), dialog->tree_view);
 
-	gtk_widget_show_all (vbox);
+	gtk_widget_show_all (dialog_vbox);
 
 	panel_toplevel_push_autohide_disabler (dialog->panel_widget->toplevel);
 	panel_widget_register_open_dialog (panel_widget,
 					   dialog->addto_dialog);
 
-	panel_addto_name_change (dialog,
-				 panel_toplevel_get_name (dialog->panel_widget->toplevel));
+	panel_addto_name_change (dialog);
 
 	return dialog;
 }
@@ -1431,7 +1396,7 @@ panel_addto_present (GtkMenuItem *item,
 		panel_addto_present_applets (dialog);
 	}
 
-	dialog->insertion_position = pd ? pd->insertion_pos : -1;
+	dialog->insert_pack_type = pd ? pd->insert_pack_type : PANEL_OBJECT_PACK_START;
 	gtk_window_set_screen (GTK_WINDOW (dialog->addto_dialog), screen);
 	gtk_window_set_default_size (GTK_WINDOW (dialog->addto_dialog),
 				     height * 8 / 7, height);
