@@ -473,8 +473,7 @@ panel_layout_append_group (GKeyFile    *keyfile,
 
 static void
 panel_layout_append_from_file_real (const char *layout_file,
-                                    int         screen_for_toplevels,
-                                    gboolean    error_fatal)
+                                    int         screen_for_toplevels)
 {
         GError    *error = NULL;
         GKeyFile  *keyfile = NULL;
@@ -522,9 +521,6 @@ out:
                 g_printerr ("Error while parsing default layout from '%s': %s\n",
                             layout_file, error->message);
                 g_error_free (error);
-
-                if (error_fatal)
-                        g_assert_not_reached ();
         }
 
         if (groups)
@@ -540,14 +536,13 @@ panel_layout_append_from_file_for_screen (const char *layout_file,
 {
         int screen_n = gdk_screen_get_number (screen);
 
-        panel_layout_append_from_file_real (layout_file, screen_n, FALSE);
+        panel_layout_append_from_file_real (layout_file, screen_n);
 }
 
 void
-panel_layout_append_from_file (const char *layout_file,
-                               gboolean    error_fatal)
+panel_layout_append_from_file (const char *layout_file)
 {
-        panel_layout_append_from_file_real (layout_file, -1, error_fatal);
+        panel_layout_append_from_file_real (layout_file, -1);
 }
 
 
@@ -889,7 +884,6 @@ panel_layout_changed_object (void)
 {
         char       **ids;
         GSList      *to_remove;
-        gboolean     loading;
         gboolean     found;
         const char  *id;
         GSList      *l;
@@ -924,8 +918,6 @@ panel_layout_changed_object (void)
 
         /* Add what appeared in the layout */
 
-        loading = FALSE;
-
         for (i = 0; ids[i] != NULL; i++) {
                 found = FALSE;
 
@@ -940,17 +932,15 @@ panel_layout_changed_object (void)
                         }
                 }
 
-                if (!found) {
+                if (!found)
                         panel_layout_load_object (ids[i]);
-                        loading = TRUE;
-                }
         }
 
         g_strfreev (ids);
 
-        /* Always do this, even if loading is FALSE: if a panel has been
-         * created, we want a do_load() to unhide it, even if there is no
-         * object to load */
+        /* Always do this, even if there is no object that got loaded: if a
+         * panel has been created, we want a do_load() to unhide it, even if
+         * there is no object to load */
         panel_object_loader_do_load (FALSE);
 }
 
@@ -1100,7 +1090,7 @@ panel_layout_load (void)
                 }
 
                 default_layout_file = panel_layout_get_default_layout_file ();
-                panel_layout_append_from_file (default_layout_file, TRUE);
+                panel_layout_append_from_file (default_layout_file);
                 g_free (default_layout_file);
 
                 toplevels = g_settings_get_strv (layout_settings,
