@@ -1,4 +1,4 @@
-/*
+/* -*- c-basic-offset: 8; indent-tabs-mode: t -*-
  * panel-run-dialog.c:
  *
  * Copyright (C) 2003 Frank Worsley <fworsley@shaw.ca>
@@ -49,6 +49,7 @@
 #include <libpanel-util/panel-gtk.h>
 #include <libpanel-util/panel-keyfile.h>
 #include <libpanel-util/panel-show.h>
+#include <libpanel-util/panel-xdg.h>
 
 #include "nothing.h"
 #include "panel-util.h"
@@ -88,20 +89,18 @@ typedef struct {
 	GCompletion      *completion;
 	
 	GSList           *add_icon_paths;
-	int	          add_icons_idle_id;
 	int	          add_items_idle_id;
 	int		  find_command_idle_id;
 	gboolean	  use_program_list;
 	gboolean	  completion_started;
 	
-	char		 *icon_path;
+	GIcon		 *gicon;
 	char             *desktop_path;
 	char		 *item_name;	
 } PanelRunDialog;
 
 enum {
 	COLUMN_ICON,
-	COLUMN_ICON_FILE,
 	COLUMN_NAME,
 	COLUMN_COMMENT,
 	COLUMN_PATH,
@@ -112,8 +111,6 @@ enum {
 
 static PanelRunDialog *static_dialog = NULL;
 
-static void panel_run_dialog_disconnect_pixmap (PanelRunDialog *dialog);
-
 #define PANEL_RUN_MAX_HISTORY 20
 
 static GtkTreeModel *
@@ -187,17 +184,15 @@ panel_run_dialog_destroy (PanelRunDialog *dialog)
 	g_slist_free (dialog->add_icon_paths);
 	dialog->add_icon_paths = NULL;
 
-	g_free (dialog->icon_path);
-	dialog->icon_path = NULL;
+	if (dialog->gicon)
+		g_object_unref (dialog->gicon);
+	dialog->gicon = NULL;
+
 	g_free (dialog->desktop_path);
 	dialog->desktop_path = NULL;
 	g_free (dialog->item_name);
 	dialog->item_name = NULL;
 
-	if (dialog->add_icons_idle_id)
-		g_source_remove (dialog->add_icons_idle_id);
-	dialog->add_icons_idle_id = 0;
-
 	if (dialog->add_items_idle_id)
 		g_source_remove (dialog->add_items_idle_id);
 	dialog->add_items_idle_id = 0;
@@ -224,8 +219,6 @@ panel_run_dialog_destroy (PanelRunDialog *dialog)
 		g_completion_free (dialog->completion);
 	dialog->completion = NULL;
 
-	panel_run_dialog_disconnect_pixmap (dialog);
-
 	if (dialog->run_settings)
 		g_object_unref (dialog->run_settings);
 	dialog->run_settings = NULL;
@@ -250,9 +243,6 @@ panel_run_dialog_set_default_icon (PanelRunDialog *dialog, gboolean set_drag)
 				      PANEL_ICON_RUN,
 				      GTK_ICON_SIZE_DIALOG);
 	
-	gtk_window_set_icon_name (GTK_WINDOW (dialog->run_dialog),
-				  PANEL_ICON_RUN);
-
 	if (set_drag)
 		gtk_drag_source_set_icon_name (dialog->run_dialog,
 					       PANEL_ICON_LAUNCHER);
@@ -260,51 +250,22 @@ panel_run_dialog_set_default_icon (PanelRunDialog *dialog, gboolean set_drag)
 
 static void
 panel_run_dialog_set_icon (PanelRunDialog *dialog,
-			   const char     *icon_path,
+			   GIcon          *gicon,
 			   gboolean        force)
 {
-	GdkPixbuf *pixbuf = NULL;
-
-	if (!force && icon_path && dialog->icon_path &&
-	    !strcmp (icon_path, dialog->icon_path))
+	if (!force && gicon && dialog->gicon &&
+	    gicon == dialog->gicon)
 		return;
 
-	g_free (dialog->icon_path);
-	dialog->icon_path = NULL;
+	if (dialog->gicon)
+		g_object_unref (dialog->gicon);
+	dialog->gicon = NULL;
 		
-	if (icon_path) {
-		GdkScreen   *screen;
-		GtkSettings *settings;
-		int          size;
-
-		screen = gtk_widget_get_screen (GTK_WIDGET (dialog->pixmap));
-		settings = gtk_settings_get_for_screen (screen);
-		gtk_icon_size_lookup_for_settings (settings,
-						   GTK_ICON_SIZE_DIALOG,
-						   &size, NULL);
-
-		pixbuf = panel_load_icon (gtk_icon_theme_get_default (),
-					  icon_path,
-					  size,
-					  size,
-					  size,
-					  NULL);
-	}
-	
-	if (pixbuf) {
-		dialog->icon_path = g_strdup (icon_path);
-
-		/* Don't bother scaling the image if it's too small.
-		 * Scaled looks worse than a smaller image.
-		 */
-		gtk_image_set_from_pixbuf (GTK_IMAGE (dialog->pixmap), pixbuf);
-
-		//FIXME: it'd be better to set an icon of the correct size,
-		//(ditto for the drag icon?)
-		gtk_window_set_icon (GTK_WINDOW (dialog->run_dialog), pixbuf);
-		
-		gtk_drag_source_set_icon_pixbuf (dialog->run_dialog, pixbuf);
-		g_object_unref (pixbuf);
+	if (gicon) {
+		dialog->gicon = g_object_ref (gicon);
+		gtk_image_set_from_gicon (GTK_IMAGE (dialog->pixmap),
+					  gicon, GTK_ICON_SIZE_DIALOG);
+		gtk_drag_source_set_icon_gicon (dialog->run_dialog, gicon);
 	} else {
 		panel_run_dialog_set_default_icon (dialog, TRUE);
 	}
@@ -649,7 +610,7 @@ panel_run_dialog_find_command_idle (PanelRunDialog *dialog)
 	GtkTreeModel *model;
 	GtkTreePath  *path;
 	char         *text;
-	char         *found_icon;
+	GIcon        *found_icon;
 	char         *found_name;
 	gboolean      fuzzy;
 	
@@ -673,23 +634,24 @@ panel_run_dialog_find_command_idle (PanelRunDialog *dialog)
 
 	do {
 		char *exec = NULL;
-		char *icon = NULL;
 		char *name = NULL;
 		char *comment = NULL;
+		GIcon *icon = NULL;
 
 		gtk_tree_model_get (model, &iter,
-				    COLUMN_EXEC,      &exec,
-				    COLUMN_ICON_FILE, &icon,
-				    COLUMN_NAME,      &name,
-				    COLUMN_COMMENT,   &comment,
+				    COLUMN_EXEC,    &exec,
+				    COLUMN_ICON,    &icon,
+				    COLUMN_NAME,    &name,
+				    COLUMN_COMMENT, &comment,
 				    -1);
 
 		if (!fuzzy && exec && icon &&
 		    fuzzy_command_match (text, exec, &fuzzy)) {
-			g_free (found_icon);
+			if (found_icon)
+				g_object_unref (found_icon);
 			g_free (found_name);
 			
-			found_icon = g_strdup (icon);
+			found_icon = g_object_ref (icon);
 			found_name = g_strdup (name);
 			
 			gtk_list_store_set (dialog->program_list_store,
@@ -711,7 +673,7 @@ panel_run_dialog_find_command_idle (PanelRunDialog *dialog)
 		}
 
 		g_free (exec);
-		g_free (icon);
+		g_object_unref (icon);
 		g_free (name);
 		g_free (comment);
 	
@@ -727,7 +689,8 @@ panel_run_dialog_find_command_idle (PanelRunDialog *dialog)
 	panel_run_dialog_set_icon (dialog, found_icon, FALSE);
 	//FIXME update dialog->program_label
 
-	g_free (found_icon);
+	if (found_icon)
+		g_object_unref (found_icon);
 	g_free (text);
 	
 	g_free (dialog->item_name);
@@ -737,62 +700,12 @@ panel_run_dialog_find_command_idle (PanelRunDialog *dialog)
 	return FALSE;
 }
 
-static gboolean
-panel_run_dialog_add_icon_idle (PanelRunDialog *dialog)
-{
-	GtkTreeIter  iter;
-	GtkTreePath *path;
-	GdkPixbuf   *pixbuf;
-	char        *file;
-	int          icon_height;
-	gboolean     long_operation = FALSE;
-	
-	do {
-		if (!dialog->add_icon_paths) {
-			dialog->add_icons_idle_id = 0;
-			return FALSE;
-		}
-
-		path = dialog->add_icon_paths->data;
-		dialog->add_icon_paths->data = NULL;
-		dialog->add_icon_paths = g_slist_delete_link (dialog->add_icon_paths,
-						              dialog->add_icon_paths);
-
-		if (!gtk_tree_model_get_iter (GTK_TREE_MODEL (dialog->program_list_store),
-					      &iter,
-					      path)) {
-			gtk_tree_path_free (path);
-			continue;
-		}
-		
-		gtk_tree_path_free (path);
-
-		gtk_tree_model_get (GTK_TREE_MODEL (dialog->program_list_store), &iter,
-				    COLUMN_ICON_FILE, &file, -1);
-
-		if (!gtk_icon_size_lookup (panel_menu_icon_get_size (), NULL, &icon_height)) {
-			icon_height = PANEL_DEFAULT_MENU_ICON_SIZE;
-		}
-
-		pixbuf = panel_make_menu_icon (gtk_icon_theme_get_default (), file, NULL, icon_height, &long_operation);
-		if (pixbuf) {
-			gtk_list_store_set (dialog->program_list_store, &iter, COLUMN_ICON, pixbuf, -1);
-			g_object_unref (pixbuf);
-		}
-		g_free (file);
-		
-	/* don't go back into the main loop if this wasn't very hard to do */
-	} while (!long_operation);
-
-	return TRUE;
-}
-
 static int
 compare_applications (GMenuTreeEntry *a,
 		      GMenuTreeEntry *b)
 {
-	return g_utf8_collate (gmenu_tree_entry_get_display_name (a),
-			       gmenu_tree_entry_get_display_name (b));
+	return g_utf8_collate (g_app_info_get_display_name ((GAppInfo*)(gmenu_tree_entry_get_app_info (a))),
+			       g_app_info_get_display_name ((GAppInfo*)(gmenu_tree_entry_get_app_info (b))));
 }
 
 static GSList *get_all_applications_from_dir (GMenuTreeDirectory *directory,
@@ -802,26 +715,23 @@ static GSList *
 get_all_applications_from_alias (GMenuTreeAlias *alias,
 				 GSList         *list)
 {
-	GMenuTreeItem *aliased_item;
-
-	aliased_item = gmenu_tree_alias_get_item (alias);
-
-	switch (gmenu_tree_item_get_type (aliased_item)) {
+	switch (gmenu_tree_alias_get_aliased_item_type (alias)) {
 	case GMENU_TREE_ITEM_ENTRY:
-		list = g_slist_append (list, gmenu_tree_item_ref (aliased_item));
+		/* pass on the reference */
+		list = g_slist_append (list, gmenu_tree_alias_get_aliased_entry (alias));
 		break;
 
-	case GMENU_TREE_ITEM_DIRECTORY:
-		list = get_all_applications_from_dir (GMENU_TREE_DIRECTORY (aliased_item),
-						      list);
+	case GMENU_TREE_ITEM_DIRECTORY: {
+		GMenuTreeDirectory *directory = gmenu_tree_alias_get_aliased_directory (alias);
+		list = get_all_applications_from_dir (directory, list);
+		gmenu_tree_item_unref (directory);
 		break;
+	}
 
 	default:
 		break;
 	}
 
-	gmenu_tree_item_unref (aliased_item);
-
 	return list;
 }
 
@@ -829,33 +739,37 @@ static GSList *
 get_all_applications_from_dir (GMenuTreeDirectory *directory,
 			       GSList             *list)
 {
-	GSList *items;
-	GSList *l;
+	GMenuTreeIter *iter;
+	GMenuTreeItemType next_type;
 
-	items = gmenu_tree_directory_get_contents (directory);
+	iter = gmenu_tree_directory_iter (directory);
 
-	for (l = items; l; l = l->next) {
-		switch (gmenu_tree_item_get_type (l->data)) {
+	while ((next_type = gmenu_tree_iter_next (iter)) != GMENU_TREE_ITEM_INVALID) {
+		switch (next_type) {
 		case GMENU_TREE_ITEM_ENTRY:
-			list = g_slist_append (list, gmenu_tree_item_ref (l->data));
+			list = g_slist_append (list, gmenu_tree_iter_get_entry (iter));
 			break;
 
-		case GMENU_TREE_ITEM_DIRECTORY:
-			list = get_all_applications_from_dir (l->data, list);
+		case GMENU_TREE_ITEM_DIRECTORY: {
+			GMenuTreeDirectory *dir = gmenu_tree_iter_get_directory (iter);
+			list = get_all_applications_from_dir (dir, list);
+			gmenu_tree_item_unref (dir);
 			break;
+		}
 
-		case GMENU_TREE_ITEM_ALIAS:
-			list = get_all_applications_from_alias (l->data, list);
+		case GMENU_TREE_ITEM_ALIAS: {
+			GMenuTreeAlias *alias = gmenu_tree_iter_get_alias (iter);
+			list = get_all_applications_from_alias (alias, list);
+			gmenu_tree_item_unref (alias);
 			break;
+		}
 
 		default:
 			break;
 		}
-
-		gmenu_tree_item_unref (l->data);
 	}
 
-	g_slist_free (items);
+	gmenu_tree_iter_unref (iter);
 
 	return list;
 }
@@ -867,15 +781,17 @@ get_all_applications (void)
 	GMenuTreeDirectory *root;
 	GSList             *retval;
 
-	tree = gmenu_tree_lookup ("applications.menu", GMENU_TREE_FLAGS_NONE);
-	gmenu_tree_set_sort_key (tree, GMENU_TREE_SORT_DISPLAY_NAME);
+	tree = gmenu_tree_new ("applications.menu", GMENU_TREE_FLAGS_SORT_DISPLAY_NAME);
+
+	if (!gmenu_tree_load_sync (tree, NULL))
+		return NULL;
 
 	root = gmenu_tree_get_root_directory (tree);
 
 	retval = get_all_applications_from_dir (root, NULL);
 
 	gmenu_tree_item_unref (root);
-	gmenu_tree_unref (tree);
+	g_object_unref (tree);
 
 	retval = g_slist_sort (retval,
 			       (GCompareFunc) compare_applications);
@@ -896,8 +812,7 @@ panel_run_dialog_add_items_idle (PanelRunDialog *dialog)
 
 	/* create list store */
 	dialog->program_list_store = gtk_list_store_new (NUM_COLUMNS,
-							 GDK_TYPE_PIXBUF,
-							 G_TYPE_STRING,
+							 G_TYPE_ICON,
 							 G_TYPE_STRING,
 							 G_TYPE_STRING,
 							 G_TYPE_STRING,
@@ -909,12 +824,15 @@ panel_run_dialog_add_items_idle (PanelRunDialog *dialog)
 	/* Strip duplicates */
 	prev_name = NULL;
 	for (l = all_applications; l; l = next) {
-		GMenuTreeEntry *entry = l->data;
-		const char     *entry_name;
+		GMenuTreeEntry  *entry = l->data;
+		const char      *entry_name;
+		GDesktopAppInfo *app_info;
+
 
 		next = l->next;
+		app_info = gmenu_tree_entry_get_app_info (entry);
 
-		entry_name = gmenu_tree_entry_get_display_name (entry);
+		entry_name = g_app_info_get_display_name (G_APP_INFO (app_info));
 		if (prev_name && entry_name && strcmp (entry_name, prev_name) == 0) {
 			gmenu_tree_item_unref (entry);
 
@@ -926,18 +844,20 @@ panel_run_dialog_add_items_idle (PanelRunDialog *dialog)
 
 	for (l = all_applications; l; l = l->next) {
 		GMenuTreeEntry *entry = l->data;
-		GtkTreeIter    iter;
-		GtkTreePath   *path;
+		GtkTreeIter     iter;
+		GtkTreePath    *path;
+		GAppInfo       *app_info;
+
+		app_info = G_APP_INFO (gmenu_tree_entry_get_app_info (entry));
 
 		gtk_list_store_append (dialog->program_list_store, &iter);
 		gtk_list_store_set (dialog->program_list_store, &iter,
-				    COLUMN_ICON,      NULL,
-				    COLUMN_ICON_FILE, gmenu_tree_entry_get_icon (entry),
-				    COLUMN_NAME,      gmenu_tree_entry_get_display_name (entry),
-				    COLUMN_COMMENT,   gmenu_tree_entry_get_comment (entry),
-				    COLUMN_EXEC,      gmenu_tree_entry_get_exec (entry),
-				    COLUMN_PATH,      gmenu_tree_entry_get_desktop_file_path (entry),
-				    COLUMN_VISIBLE,   TRUE,
+				    COLUMN_ICON,    g_app_info_get_icon (app_info),
+				    COLUMN_NAME,    g_app_info_get_display_name (app_info),
+				    COLUMN_COMMENT, g_app_info_get_description (app_info),
+				    COLUMN_EXEC,    g_app_info_get_executable (app_info),
+				    COLUMN_PATH,    gmenu_tree_entry_get_desktop_file_path (entry),
+				    COLUMN_VISIBLE, TRUE,
 				    -1);
 
 		path = gtk_tree_model_get_path (GTK_TREE_MODEL (dialog->program_list_store), &iter);
@@ -963,7 +883,7 @@ panel_run_dialog_add_items_idle (PanelRunDialog *dialog)
 	column = gtk_tree_view_column_new ();
 	gtk_tree_view_column_pack_start (column, renderer, FALSE);
 	gtk_tree_view_column_set_attributes (column, renderer,
-                                             "pixbuf", COLUMN_ICON,
+                                             "gicon", COLUMN_ICON,
                                              NULL);
         
 	renderer = gtk_cell_renderer_text_new ();
@@ -976,11 +896,6 @@ panel_run_dialog_add_items_idle (PanelRunDialog *dialog)
 
 	dialog->add_icon_paths = g_slist_reverse (dialog->add_icon_paths);
 
-	if (!dialog->add_icons_idle_id)
-		dialog->add_icons_idle_id =
-			g_idle_add_full (G_PRIORITY_LOW, (GSourceFunc) panel_run_dialog_add_icon_idle,
-					 dialog, NULL);
-
 	dialog->add_items_idle_id = 0;					 
 	return FALSE;
 }
@@ -1088,7 +1003,17 @@ program_list_selection_changed (GtkTreeSelection *selection,
 	g_free (temp);
 
 	temp = panel_key_file_get_locale_string (key_file, "Icon");
-	panel_run_dialog_set_icon (dialog, temp, FALSE);
+	if (!PANEL_GLIB_STR_EMPTY (temp)) {
+		GIcon *gicon;
+
+		stripped = panel_xdg_icon_remove_extension (temp);
+		gicon = g_themed_icon_new (stripped);
+		panel_run_dialog_set_icon (dialog, gicon, FALSE);
+		g_object_unref (gicon);
+		g_free (stripped);
+	} else {
+		panel_run_dialog_set_icon (dialog, NULL, FALSE);
+	}
 	g_free (temp);
 	
 	temp = panel_key_file_get_locale_string (key_file, "Comment");
@@ -1769,6 +1694,7 @@ panel_run_dialog_create_desktop_file (PanelRunDialog *dialog)
 	gboolean  exec = FALSE;
 	char     *text;
 	char     *name;
+	char     *icon;
 	char     *disk;
 	char     *scheme;
 	char     *save_uri;
@@ -1815,10 +1741,14 @@ panel_run_dialog_create_desktop_file (PanelRunDialog *dialog)
 	panel_key_file_set_boolean (key_file, "Terminal",
 				    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->terminal_checkbox)));
 
-	if (dialog->icon_path)
+	icon = NULL;
+	if (dialog->gicon)
+		icon = panel_util_get_icon_name_from_g_icon (dialog->gicon);
+	if (icon != NULL) {
 		panel_key_file_set_locale_string (key_file, "Icon",
-						  dialog->icon_path);
-	else
+						  icon);
+		g_free (icon);
+	} else
 		panel_key_file_set_locale_string (key_file, "Icon",
 						  PANEL_ICON_LAUNCHER);
 
@@ -1861,46 +1791,12 @@ pixmap_drag_data_get (GtkWidget          *run_dialog,
 	}
 }
 
-static void
-panel_run_dialog_style_updated (GtkWidget      *widget,
-                                PanelRunDialog *dialog)
-{
-  if (dialog->icon_path) {
-	  char *icon_path;
-
-	  icon_path = g_strdup (dialog->icon_path);
-	  panel_run_dialog_set_icon (dialog, icon_path, TRUE);
-	  g_free (icon_path);
-  }
-}
-
-static void
-panel_run_dialog_screen_changed (GtkWidget      *widget,
-				 GdkScreen      *prev_screen,
-				 PanelRunDialog *dialog)
-{
-  if (dialog->icon_path) {
-	  char *icon_path;
-
-	  icon_path = g_strdup (dialog->icon_path);
-	  panel_run_dialog_set_icon (dialog, icon_path, TRUE);
-	  g_free (icon_path);
-  }
-}
-
 static void
 panel_run_dialog_setup_pixmap (PanelRunDialog *dialog,
 			       GtkBuilder     *gui)
 {
 	dialog->pixmap = PANEL_GTK_BUILDER_GET (gui, "icon_pixmap");
 	
-	g_signal_connect (dialog->pixmap, "style-updated",
-			  G_CALLBACK (panel_run_dialog_style_updated),
-			  dialog);
-	g_signal_connect (dialog->pixmap, "screen-changed",
-			  G_CALLBACK (panel_run_dialog_screen_changed),
-			  dialog);
-
 	g_signal_connect (dialog->run_dialog, "drag_data_get",
 			  G_CALLBACK (pixmap_drag_data_get),
 			  dialog);
@@ -1934,7 +1830,9 @@ panel_run_dialog_new (GdkScreen  *screen,
 	panel_run_dialog_setup_program_list  (dialog, gui);
 	panel_run_dialog_setup_list_expander (dialog, gui);
 
-	panel_run_dialog_set_default_icon    (dialog, FALSE);
+	gtk_window_set_icon_name (GTK_WINDOW (dialog->run_dialog),
+				  PANEL_ICON_RUN);
+	panel_run_dialog_set_default_icon (dialog, FALSE);
 
 	g_signal_connect (dialog->run_settings, "changed::"PANEL_RUN_ENABLE_LIST_KEY,
 			  G_CALLBACK (panel_run_dialog_update_program_list), dialog);
@@ -1959,17 +1857,6 @@ panel_run_dialog_new (GdkScreen  *screen,
 	return dialog;
 }
 
-static void
-panel_run_dialog_disconnect_pixmap (PanelRunDialog *dialog)
-{
-	g_signal_handlers_disconnect_by_func (dialog->pixmap,
-			                      G_CALLBACK (panel_run_dialog_style_updated),
-			                      dialog);
-	g_signal_handlers_disconnect_by_func (dialog->pixmap,
-			                      G_CALLBACK (panel_run_dialog_screen_changed),
-			                      dialog);
-}
-
 static void
 panel_run_dialog_static_dialog_destroyed (PanelRunDialog *dialog)
 {
